import requests
from requests_html import HTMLSession

from command_system import RollbotResponse, RollbotFailure, as_plugin, pop_arg
from config import API_KEY
from util import upload_image

@as_plugin
def inspire(db, message):
    argstr = message.raw_args

    if argstr is not None and "explain" in argstr:
        return RollbotResponse(
            message,
            txt="Using the command !inspire will request an inspiration image from http://inspirobot.me/"
        )

    try:
        url = requests.get("http://inspirobot.me/api?generate=true").text
        return RollbotResponse(message, img=url)
    except ConnectionError as e:
        return RollbotResponse(
            message,
            failure_reason=RollbotFailure.SERVICE_DOWN,
            failure_notes={
                "explain": "Could not reach inspirobot.",
                "exception": e
            }
        )


@as_plugin
def shield(db, message):
    data = """\
------RollbotBoundary\r\n\
Content-Disposition: form-data; name=\"blazon\"\r\n\
\r\n\
%s\r\n\
------RollbotBoundary\r\n\
Content-Disposition: form-data; name=\"asfile\"\r\n\
\r\n\
1\r\n\
------RollbotBoundary\r\n\
Content-Disposition: form-data; name=\"palette\"\r\n\
\r\n\
wappenwiki\r\n\
------RollbotBoundary\r\n\
Content-Disposition: form-data; name=\"effect\"\r\n\
\r\n\
flat\r\n\
------RollbotBoundary
""" % message.raw_args
    # TODO - this should probably have a random number as part of the boundary

    try:
        r = requests.post(
            "https://drawshield.net/include/drawshield.php",
            headers={
                'Content-Type': "multipart/form-data; boundary=----RollbotBoundary",
                'Cache-Control': "no-cache",
            },
            data=data
        )
    except ConnectionError as e:
        return RollbotResponse(
            message,
            failure_reason=RollbotFailure.SERVICE_DOWN,
            failure_notes={
                "explain": "Could not reach DrawShield.",
                "exception": e,
            }
        )

    success, result = upload_image(API_KEY, r.content)
    if success:
        return RollbotResponse(message, img=result)
    else:
        return RollbotResponse(
            message,
            failure_reason=RollbotFailure.SERVICE_DOWN,
            failure_notes=result
        )


@as_plugin
def scp(db, message):
    number, _ = pop_arg(message.raw_args)
    try:
        number = int(number.strip())
    except ValueError as e:
        return RollbotResponse(
            message,
            failure_reason=RollbotFailure.INVALID_ARGUMENTS,
            failure_notes={
                "explain": "Could not parse argument %s into integer." % number,
                "exception": e,
            }
        )

    page_url = "http://www.scp-wiki.net/scp-%03d" % number
    series_url = "http://www.scp-wiki.net/scp-series"
    series = (number // 1000) + 1
    if series != 1:
        series_url += "-" + str(series)

    sesh = HTMLSession()

    r = sesh.get(page_url)
    r2 = sesh.get(series_url)

    ps = r.html.find("p")
    lis = r2.html.find("li")

    object_class = "Error retrieving object class!"
    for p in ps:
        if "Object Class" in p.text:
            object_class = p.text
            break

    query = "SCP-%03d" % number
    flavortext = "Error retrieving title!"
    for li in lis:
        if query in li.text:
            flavortext = li.text.split("-", 2)[-1].strip()
            break

    response = "Item # %d\n" % number
    response += object_class + "\n"
    response += flavortext + "\n"
    response += page_url

    return RollbotResponse(message, txt=response)